/*
**  Copyright (c) 2003,2004,2005 Andrea Luzzardi <scox@gentoo.org>
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include "pam.h"
#include "parser.h"
#include "conf.h"

void replace_char (char *str, char source, char dest)
{
	size_t i = 0;

	while (str[i]) {
		if (str[i] == source)
			str[i] = dest;
		++i;
	}
}

char *getline_until (FILE *f, char delim)
{
	size_t c, i = 0, start = 0, skip = 0;

	char *buf = NULL;
	while ((c = fgetc(f)) != EOF) {
		/* skip comments */

		if (skip && c != '\n')
			continue;

		if (skip && c == '\n') {
			skip = 0;
			continue;
		}

		if (!start && c == '#') {
			skip = 1;
			continue;
		}

		/* skip spaces before text */
		if (!start && c == ' ')
			continue;
		++start;

		buf = realloc (buf, i+1);
		if (c == delim) {
			buf [i] = '\0';
			return buf;
		}
		buf[i] = c;
		++i;
	}

	return buf;
}

char *get_from_to (const char *line, char start, char stop)
{
	size_t i, len, copied = 0, started = 0;
	char *buf = malloc (1);

	if (!buf)
		return NULL;

	if (!start)
		started = 1;

	len = strlen(line);
	for (i=0; i <= len; ++i)
	{
		if (line[i] == stop) {
			buf[copied] = '\0';
			return buf;
		}

		if (line[i] == start) {
			started = 1;
			continue;
		}
		if (!started)
			continue;

		buf[copied] = line[i];
		++copied;
		buf = realloc (buf, copied+1);
	}

	return NULL;
}

char *get_param (FILE *f, char *param, char delim)
{
	char *line;
	char *val;

	while ((line = getline_until(f, delim)))
	{
		val = getline_until(f, '\n');
		if (!strcmp(param, line)) {
			free (line);
			return val;
		}
		free (val);
		free (line);
	}

	return NULL;
}

int check_param (FILE *f, char *param, char *value, char delim)
{
	char *val;

	val = get_param (f, param, delim);
	if (!val)
		return 0;

	if (!strcmp (value, val)) {
		free (val);
		return 1;
	}

	free (val);
	return 0;
}

char *insert_before (char *src, char *dst)
{
	char *buf;
	size_t len = strlen(src) + strlen(dst) + 1;

	buf = malloc (len);
	if (buf) {
		strcpy(buf, src);
		strcat(buf, dst);
	}

	return buf;
}

void drop_split (char **split)
{
	char **temp = split;

	while (*temp) {
		free (*temp);
		++temp;
	}

	free (split);
}

char **split (char *line, int delim)
{
	char *buf;
	char **ret;
	size_t i, len, copied = 0, nelem = 0;

	ret = malloc (sizeof(char*));
	buf = malloc (sizeof(char));

	ret[0] = NULL;
	buf[0] = '\0';

	len = strlen(line);
	for (i = 0; i <= len; ++i) {
		if ((line[i] == delim) || (line[i] == '\0')) {
			/* we found the delimiter, or the end of line */

			/* if empty, skip */
			if (!copied)
				continue;

			/* put the buffer into the array */
			buf[copied] = '\0';
			ret[nelem] = buf;

			/* reset the counter */
			copied = 0;

			/* increase the array size */
			++nelem;
			ret = realloc (ret, (nelem+1) * sizeof(char*));
			ret[nelem] = NULL;

			/* reset buf */
			buf = malloc (sizeof(char));
			buf[0] = '\0';

			/* go ahead */
			continue;
		}

		buf[copied] = line[i];
		++copied;
		buf = realloc (buf, copied+1);
	}

	return ret;
}
