/*
**  Copyright (c) 2003,2004,2005 Andrea Luzzardi <scox@gentoo.org>
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include "usbadm.h"
#include <device.h>
#include <parser.h>

int addserial (int ac, char **av)
{
	char *serial = NULL;
	FILE *f;

	if (ac >= 3)
		serial = av[2];

	if (!serial) {
		char *file, *tmp;

		while ((file = find_proc_file ())) {
			if (!(f = fopen (file, "r"))) {
				free (file);
				continue;
			}
			printf ("[+] procfile: %s\n", file);

			serial = get_param(f, "Serial Number", ':');

			if (serial &&
			    !strcmp (serial, "None")) {
				free (serial);
				serial = NULL;
			}

			if (!serial) {
				printf ("[-] serial number: none\n");

				/* fall back to GUID */
				if (!(serial = get_param (f, "GUID", ':'))) {
					fclose (f);
					free (file);
					printf ("[-] GUID: none\n");
					continue;
				}

				printf ("[+] GUID: %s\n", serial);
				tmp = insert_before ("{G} ", serial);
				free (serial);
				serial = tmp;
			} else {
				printf ("[+] serial number: %s\n", serial);
				tmp = insert_before ("{S} ", serial);
				free (serial);
				serial = tmp;
			}

			if (!serial) {
				fclose (f);
				free (file);
				break;
			}

			printf ("[+] auth number: %s\n", serial);

			if (!check_param (f, "Attached", "Yes", ':')) {
				fclose (f);
				free (file);
				free (serial);
				serial = NULL;
				printf ("[-] attached: no\n");
				continue;
			}
			fclose (f);
			free (file);
			printf ("[+] attached: yes\n");
			break;
		}
	}

	if (!serial) {
		printf ("[-] Cannot retrieve the serial number\n");
		return 1;
	}

	if (!(f = fopen (usb_conf.snfile, "a"))) {
		printf ("cannot open file %s\n", usb_conf.snfile);
		free (serial);
		return 1;
	}

	printf ("[!] Allowing serial number %s...", serial);
	fflush (stdout);

	if (fputs (serial, f) == EOF) {
		printf ("[-] cannot write file %s\n", usb_conf.snfile);
		fclose (f);
		free (serial);
		return 1;
	}
	fputc ('\n', f);
	fclose (f);
	free (serial);

	printf ("done.\n");

	return 0;
}
