/*
**  Copyright (c) 2003,2004,2005 Andrea Luzzardi <scox@gentoo.org>
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <stdio.h>
#include <stdlib.h>
#include <shadow.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>

#include <conf.h>
#include "usbadm.h"

struct action actions[] = {
	{ "help", "display action description and usage.", "[action]", help},
	{ "keygen", "generate per host private/public DSA key pair", "<mntpoint> <user> <bits>",  keygen },
	{ "cipher", "manage the private key encryption (encrypt/decrypt/change password)", "<mntpoint> <user> [algorithm]", cipher },
	{ "addserial", "grants the specified serial number.", "[serial number]", addserial},
	{ NULL, NULL, NULL, NULL }
};

struct action *find_action (char *name)
{
	struct action *action = actions;
	while (action->name) {
		if (!strcmp (action->name, name)) {
			return action;
		}
		action++;
	}
	return NULL;
}

int main (int ac, char **av)
{
	struct action *action = NULL;

	if (ac < 2) {
		printf ("usage: %s <action> [arguments]\n", av[0]);
		printf ("See %s help\n", av[0]);
		return 1;
	}

	parse_args (ac, (const char **)av);
	put_default_values ();

	if (!(action = find_action (av[1]))) {
		printf ("%s: unknown action\n", av[1]);
		return 1;
	}
	return action->handler(ac, av);
}
