/*
**  Copyright (c) 2003,2004,2005 Andrea Luzzardi <scox@gentoo.org>
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/


#include "usbhotplug.h"

int conv (int n, const struct pam_message **msg,
	struct pam_response **resp, void *data)
{
	/* Okay, as we're called by the kernel because of a hotplug
	 * event, we can't prompt the user or tell him anything.
	 * So we'll just ignore the conversation. */

	return PAM_BUF_ERR;
}

int pam_do_auth (void)
{
	int status = 0;
	struct pam_conv c;
	pam_handle_t *pamh = NULL;

	c.conv = &conv;

	if (pam_start ((const char*)SERVICE, (const char*)config.auth, &c, &pamh)
		!= PAM_SUCCESS) {
		DEBUG ("Cannot start PAM for the user %s\n", config.auth);
		return 0;
	}

	if (pam_set_item (pamh, PAM_TTY, ":0.0") != PAM_SUCCESS) {
		DEBUG ("Warning: Cannot set tty to :0.0\n");
	}

	status = pam_authenticate (pamh, 0);

	pam_end (pamh, status);

	if (status != PAM_SUCCESS)
		return 0;

	return 1;
}
