/*
**  Copyright (c) 2003,2004,2005 Andrea Luzzardi <scox@gentoo.org>
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/


#include "usbhotplug.h"
#include "auth.h"
#include "event.h"

struct hotplug_config config;

struct _expected_config expected_config[] = {
	{"ENABLE"	,	&config.enable,		1	},
	{"AUTH"		,	&config.auth,		1	},
	{"USER"		,	&config.user,		0	},
	{"START"	,	&config.start,		1	},
	{"STOP"		,	&config.stop,		1	},
	{"DELAY"	,	&config.sleep,		0	},
	{"LOG_FILE"	,	&config.logfile,	0	},
	{NULL		,	NULL,			0	}
};

int load_config (void)
{
	struct _expected_config *config_mem = expected_config;
	FILE *f;

	if (!(f = fopen (CONF_FILE, "r"))) {
		DEBUG ("Cannot open configuration file (%s)\n", CONF_FILE);
		return 0;
	}

	while (config_mem->name) {
		(*config_mem->value) = get_param (f, config_mem->name, '=');

		if (!(*config_mem->value) && config_mem->required) {
			DEBUG ("Missing required argument [%s], aborting.\n", 
				config_mem->name);
			return 0;
		}

		DEBUG ("Got %s = %s\n", config_mem->name, (*config_mem->value) ? (*config_mem->value) : "none");
		
		rewind (f);
		config_mem++;
	}

	return 1;
}

void init_log (void)
{
	if (!(config.logfile))
		return;

	if (!(log_stream = fopen (config.logfile, "a"))) {
		log_stream = stderr;
		DEBUG ("Cannot open log file %s\n", config.logfile);
		return;
	}

	DEBUG ("--- LOG STARTED ---\n");
}

void fini_log (void)
{
	if (!config.logfile)
		return;

	fclose (log_stream);
}

int main (int ac, char **av)
{
	int status = 0;
	int delay = 0;

	struct device_info info;

	memset (&info, '\0', sizeof (struct device_info));
	memset (&config, '\0', sizeof (struct hotplug_config));

	log_stream = stderr;

	put_default_values();

	if (ac < 2) {
		fprintf (stderr, "%s: don't run this manually.\n", av[0]);
		return 1;
	}

	if (!load_config()) {
		DEBUG ("Unable to load configuration.\n");
		return 1;
	}

	if (strcmp (av[1], "block") != 0) {
		DEBUG ("Discarding non-usb device hotplug event [%s]\n", av[1]);
		return 0;
	}

	init_log();

	DEBUG ("Received a hotplug event\n");

	if (!atoi(config.enable)) {
		DEBUG ("usbhotplug not enabled, aborting.\n");
		DEBUG ("Set ENABLE=1 to enable.\n");
		return 0;
	}

	/* Who cares we cannot retrieve the informations...
	 * if we cannot authenticate the user, then we must lock down.
	 * In short, just display a warning and try to authenticate the user.
	*/

	if (!retrieve_info (&info))
		DEBUG ("Cannot retrieve device informations\n");

	/* Give it some time to do its job... */
	delay = atoi (config.sleep);

	if (delay)
		sleep (delay);

	status = pam_do_auth();

	DEBUG ("Access %s for user %s, starting the handler\n",
		status ? "granted" : "denied", config.auth);

	handle_event (status);

	fini_log();

	return 0;
}
